Package: randRotation
Title: Random Rotation Methods for High Dimensional Data with Batch
        Structure
Version: 1.22.0
Authors@R: 
    person(given = "Peter",
           family = "Hettegger",
           role = c("aut", "cre"),
           email = "p.hettegger@gmail.com",
           comment = c(ORCID = "0000-0001-8557-588X"))
Description: A collection of methods for performing random rotations on high-dimensional, normally distributed data (e.g. microarray or RNA-seq data) with batch structure.
    The random rotation approach allows exact testing of dependent test statistics with linear models following arbitrary batch effect correction methods.
License: GPL-3
biocViews: Software, Sequencing, BatchEffect, BiomedicalInformatics,
        RNASeq, Preprocessing, Microarray, DifferentialExpression,
        GeneExpression, Genetics, MicroRNAArray, Normalization,
        StatisticalMethod
Encoding: UTF-8
RdMacros: Rdpack
Imports: methods, graphics, utils, stats, Rdpack (>= 0.7)
RoxygenNote: 7.2.3
Suggests: knitr, BiocParallel, lme4, nlme, rmarkdown, BiocStyle,
        testthat (>= 2.1.0), limma, sva
VignetteBuilder: knitr
URL: https://github.com/phettegger/randRotation
BugReports: https://github.com/phettegger/randRotation/issues
git_url: https://git.bioconductor.org/packages/randRotation
git_branch: RELEASE_3_22
git_last_commit: 4c9448f
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:56:56 UTC; biocbuild
Author: Peter Hettegger [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8557-588X>)
Maintainer: Peter Hettegger <p.hettegger@gmail.com>
