\name{randRotation-defunct}
\alias{df_estimate}
\title{Defunct functions in package \sQuote{randRotation}}
\description{These functions are defunct and no longer available.}

\details{
  Defunct functions are: \code{df_estimate}
}
